package dev.svitan

import io.github.cdimascio.dotenv.dotenv
import io.ktor.server.application.*
import io.ktor.server.engine.*
import io.ktor.server.netty.*
import dev.svitan.plugins.configureAdministration
import dev.svitan.plugins.configureDatabases
import dev.svitan.plugins.configureErrors
import dev.svitan.plugins.configureHTTP
import dev.svitan.plugins.configureMonitoring
import dev.svitan.plugins.configureRouting
import dev.svitan.plugins.configureSecurity
import dev.svitan.plugins.configureSerialization

fun main() {
    embeddedServer(
        Netty,
        port = 8080,
        host = "0.0.0.0",
        module = Application::module
    ).start(wait = true)
}

fun Application.module() {
    val dotenv = dotenv()

    configureHTTP()
    configureErrors()
    configureRouting()
    configureMonitoring()
    configureSerialization()
    configureAdministration()
    configureSecurity(dotenv)
    configureDatabases(dotenv)
}
