package dev.svitan.plugins

import dev.svitan.services.ExecutorService
import dev.svitan.services.ActionService
import dev.svitan.services.AuthService
import io.github.cdimascio.dotenv.Dotenv
import io.ktor.server.application.*
import org.jetbrains.exposed.sql.Database

fun Application.configureDatabases(dotenv: Dotenv) {
    val dbHost = dotenv["DB_HOST"] ?: throw Exception("DB_HOST not found")
    val dbPort = dotenv["DB_PORT"] ?: throw Exception("DB_PORT not found")
    val dbName = dotenv["DB_NAME"] ?: throw Exception("DB_NAME not found")
    val dbUser = dotenv["DB_USER"] ?: throw Exception("DB_USER not found")
    val dbPassword = dotenv["DB_PASSWORD"] ?: throw Exception("DB_PASSWORD not found")

    Database.connect(
        url = "jdbc:postgresql://${dbHost}:${dbPort}/${dbName}",
        driver = "org.postgresql.Driver",
        user = dbUser,
        password = dbPassword
    )

    AuthService.init()
    ActionService.init()
    ExecutorService.init(dotenv)
}
