package dev.svitan.routes

import dev.svitan.plugins.AuthorizationException
import dev.svitan.services.ActionKind
import dev.svitan.services.ActionService
import dev.svitan.services.AuthService
import dev.svitan.services.ExecutorService
import dev.svitan.services.NewActionDTO
import dev.svitan.services.RunActionDTO
import dev.svitan.services.UpdateActionDTO
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.auth.*
import io.ktor.server.plugins.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import java.util.*

fun Application.routeAction() {
    routing {
        authentication {
            get("/action") {
                call.respond(ActionService.readAll())
            }

            put("/action") {
                val action = call.receive<NewActionDTO>()
                call.respond(HttpStatusCode.Created, ActionService.create(action))
            }

            get("/action/{id}") {
                val idRaw = call.parameters["id"] ?: throw BadRequestException("Invalid id")
                val id = UUID.fromString(idRaw)
                val action = ActionService.read(id) ?: throw NotFoundException()
                call.respond(action)
            }

            post("/action/{id}") {
                val idRaw = call.parameters["id"] ?: throw BadRequestException("Invalid id")
                val id = UUID.fromString(idRaw)

                val action = ActionService.read(id) ?: throw NotFoundException()
                val run = call.receive<RunActionDTO>()
                AuthService.validate(
                    UUID.fromString(action.authId),
                    run.pin,
                    run.key
                ) ?: throw AuthorizationException("Invalid pin or key")

                if (action.kind == ActionKind.TEXT) {
                    ExecutorService.executeSource(id, action.source)
                } else {
                    ExecutorService.executeFile(id)
                }

                call.respond(HttpStatusCode.NoContent)
            }

            patch("/action/{id}") {
                val idRaw = call.parameters["id"] ?: throw BadRequestException("Invalid id")
                val id = UUID.fromString(idRaw)
                val action = call.receive<UpdateActionDTO>()
                call.respond(ActionService.update(id, action))
            }

            delete("/action/{id}") {
                val idRaw = call.parameters["id"] ?: throw BadRequestException("Invalid id")
                val id = UUID.fromString(idRaw)
                ActionService.delete(id)
                call.respond(HttpStatusCode.NoContent)
            }
        }
    }
}
